// Copyright (c) 2014, The Shelf Bind project authors.
// Please see the AUTHORS file for details.
// All rights reserved. Use of this source code is governed by
// a BSD 2-Clause License that can be found in the LICENSE file.

library shelf_bind.annotation;
import 'dart:io';

/**
 * An annotation that indicates that a parameter should be bound to the request
 * body. The body will be parsed into an object of the specified [type].
 *
 * The default format for the body is JSON. In the future this will be
 * able to be overriden by specifying a value for [format]
 */
class RequestBody {
  final BodyFormat format;
  final Type type;

  const RequestBody({ this.format: BodyFormat.JSON, this.type });
}

/**
 * An enum specifying the format for a [Request] body
 */
class BodyFormat {
  final String name;

  const BodyFormat._internal(this.name);

  static const BodyFormat JSON = const BodyFormat._internal("JSON");
}


/**
 * Annotation to tweak default creating of responses
 */
class ResponseHeaders {
  /// The HttpStatus to use if the handler returns normally (no exception thrown).
  /// Default is OK. Most common use would be to return CREATED instead for
  /// certain operations
  final int successStatus;

  /// Set to [true] if the location header should be populated automatically.
  /// Default is false.
  /// If [true] the location header is populated to the incoming request url,
  /// which is normally correct if creating via a PUT (but not POST)
  final bool setLocation;


  const ResponseHeaders({ this.successStatus: HttpStatus.OK,
    this.setLocation: false});

  /// Return a CREATED status code and populate the location automatically
  const ResponseHeaders.created() :
    this(successStatus: HttpStatus.CREATED, setLocation: true);
}

//class ResponseX {
//  final RequestBody body;
//  final ResponseHeaders headers;
//
//  const ResponseX({ this.body: const RequestBody(),
//    this.headers: ResponseHeaders.OK });
//}