// Copyright (c) 2014, The Shelf Bind project authors.
// Please see the AUTHORS file for details.
// All rights reserved. Use of this source code is governed by
// a BSD 2-Clause License that can be found in the LICENSE file.

library shelf_bind.convert;

import 'dart:convert';

// TODO: this should be moved to a standalone package on pub

convertTo(Type type, String value) =>
  _codec(type).decode(value);

String convertFrom(Type type, dynamic value) =>
    _codec(type).encode(value);


class NumCodec extends Codec<num, String> {
  @override
  final Converter<String, num> decoder = const _StringToNumConverter();

  @override
  final Converter<num, String> encoder = const _NumToStringConverter();

  const NumCodec();
}

class _StringToNumConverter extends Converter<String, num> {
  const _StringToNumConverter();

  @override
  num convert(String input) => num.parse(input);
}

class _NumToStringConverter extends Converter<num, String> {
  const _NumToStringConverter();

  @override
  String convert(num input) => input.toString();
}


final Map<Type, Codec<dynamic, String>> _stringCodecs = {
  num: const NumCodec(),
  int: const NumCodec(), // TODO: avoid by checking 'is'
  double: const NumCodec()
};

_codec(Type type) {
  final codec = _stringCodecs[type];
  if (codec == null) {
    throw new ArgumentError('no codec for type $type');
  }
  return codec;
}